package de.fraunhofer.iem.dataprovider.kpi.entity

import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeChildResponseDto
import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeResponseDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import jakarta.persistence.*
import org.hibernate.annotations.JdbcTypeCode
import org.hibernate.type.SqlTypes
import java.sql.Timestamp
import java.util.*

@Entity
@Table(name = "kpi")
class KPIEntity {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    @JoinColumn(name = "repository_id")
    var repository: RepositoryEntity? = null

    @OneToMany(mappedBy = "from", cascade = [CascadeType.MERGE, CascadeType.REMOVE], orphanRemoval = true)
    var hierarchyEdges: MutableSet<KPIHierarchyEdgeEntity> = mutableSetOf()

    @Column(name = "score")
    var score: Int? = null

    @Column(name = "display_score")
    var displayScore: String? = null

    @Column(name = "min")
    var min: Int? = null

    @Column(name = "max")
    var max: Int? = null

    @Column(name = "is_root")
    var isRoot: Boolean? = null

    @Column(name = "is_empty")
    var isEmpty: Boolean? = null

    @Column(name = "name")
    var name: String? = null

    @Column(name = "description", columnDefinition = "TEXT")
    var description: String? = null

    @JdbcTypeCode(SqlTypes.TIMESTAMP)
    @Column(name = "timestamp")
    var timestamp: Timestamp? = null

    fun toResultDto(): KPITreeResponseDto {
        return this.buildKPITree(this)
    }

    // Helper function to build the KPI tree
    private fun buildKPITree(rootKPI: KPIEntity): KPITreeResponseDto {
        val childTrees = rootKPI.hierarchyEdges.map { edge ->
            KPITreeChildResponseDto(buildKPITree(edge.to!!), edge.weight!!)
        }
        return KPITreeResponseDto(
            rootKPI.id!!,
            rootKPI.score!!,
            rootKPI.name!!,
            rootKPI.description!!,
            rootKPI.isRoot!!,
            rootKPI.displayScore,
            childTrees,
            this.repository!!.id!!,
            this.isEmpty!!
        )
    }
}
